
#!/usr/bin/env python
from gimpfu import *
import random
from random import randint

def filling(inImage, inDrawable, inColourE, inColourS,
            inSearch, inMethod,
            inPalette, inGradient, pnfilter,
            workOnCopy) :

    pdb.gimp_context_push
    pdb.gimp_context_set_sample_transparent (True)
    
    if workOnCopy==True:
        outImage = pdb.gimp_image_duplicate (inImage)
    else:
        outImage = inImage
    outDrawable = outImage.layers[0]
    currentFG = pdb.gimp_context_get_foreground()
    #pdb.gimp_message("Hello from my FILLING filter!")
    width = outImage.width
    height = outImage.height
    #pdb.gimp_message("Your input image dimensions are:  "
    #                 +"width="+str(width)+", height="+str(height))
    
    (Re, Ge, Be, Ae)=inColourE
    #pdb.gimp_message("Your chosen colour to replace is:  "
    #                 +"R="+str(Re)+", G="+str(Ge)+", B="+str(Be)+", A="+str(Ae))
    
    
    n=0
    steps=max(1,int(width+height)/300)
    #pdb.gimp_message("Step = "+str(steps))
    widthpoints=width/steps
    heightpoints=height/steps
    
    for w in range(widthpoints):  
      for h in range(heightpoints):
        
        pdb.gimp_context_set_foreground(currentFG)  # it's changed during processing 
        
        if inSearch==0: # random
            x = randint(0,width-1)
            y = randint(0,height-1)
        
        if inSearch==1: # sequential
            x = w*steps
            y = h*steps
            
        #pdb.gimp_message("These cohordinates are:  "+"x="+str(x)+", y="+str(y))
    
        (R,G,B,A)=pdb.gimp_image_pick_color(outImage, outDrawable, x, y, False, False, 0)
        #pdb.gimp_message("Picked colour is:  "
        #             +"R="+str(R)+", G="+str(G)+", B="+str(B)+", A="+str(A))
    
        if (R==Re) and (G==Ge) and (B==Be) and (A==Ae):    # important to add also the transparency component
            pdb.gimp_image_select_contiguous_color(outImage, CHANNEL_OP_REPLACE, outDrawable, x, y)
            # recolouring according to user choice
            if inMethod==0: # random colours (light)
                Rr = randint(100,255)
                Gr = randint(100,255)
                Br = randint(100,255)
                Ar = 255
                #pdb.gimp_message("Generated colour is:  "
                #             +"Rr="+str(Rr)+", Gr="+str(Gr)+", Br="+str(Br)+", Ar="+str(Ar))

            
            if inMethod==4: # add a random pattern layer
                (countP, namesP) = pdb.gimp_patterns_list (pnfilter)
                #pdb.gimp_message("countP = "+str(countP))
                thisone = random.randint (0, countP-1)
                pdb.gimp_context_set_pattern (namesP[thisone])
                pdb.gimp_edit_fill (outDrawable, FILL_PATTERN)
             
            else:
              if inMethod==1: # random colours from chosen palette
                LC=[]
                (numcolP, LC)=pdb.gimp_palette_get_colors(inPalette)
                Ic = randint(0,numcolP-1)
                (Rr,Gr,Br,Ar)=LC[Ic]
              if inMethod==2: # random colours from chosen gradient
                LC=[]
                (numcolP, LC)=pdb.gimp_gradient_get_uniform_samples(inGradient, 65, False)
                #pdb.gimp_message("numcolP="+str(numcolP))
                #pdb.gimp_message("LC="+str(LC))
                Ic = randint(0,259)
                Ic = int(Ic/4)*4   #try
                #pdb.gimp_message("Ic="+str(Ic))
                Rr=int(float(LC[Ic+0])*256.0)     
                Gr=int(float(LC[Ic+1])*256.0)
                Br=int(float(LC[Ic+2])*256.0)
                Ar=int(float(LC[Ic+3])*256.0)
                #pdb.gimp_message("random pointer="+str(Ic)+": red="+str(Rr)+" green="+str(Gr)+" blue="+str(Br))
              if inMethod==3: # random grays
                Rr = randint(60,200)
                Gr = Rr
                Br = Rr
                Ar = 255
              pdb.gimp_context_set_foreground((Rr,Gr,Br,Ar))  # set replacing colour
              pdb.gimp_edit_fill(outDrawable, FILL_FOREGROUND)
             
    
    # now fill all remaining areas by a fixed replacing colour
    pdb.gimp_image_select_color(outImage, CHANNEL_OP_REPLACE, outDrawable, inColourE)
    if pdb.gimp_selection_is_empty == False:    # important because if empty the whole drawable is filled!
        pdb.gimp_context_set_foreground(inColourS)  # set replacing colour
        pdb.gimp_edit_fill(outDrawable, FILL_FOREGROUND)
     
    
    pdb.gimp_selection_none(outImage)
    pdb.gimp_context_set_foreground(currentFG)
    pdb.gimp_context_pop
    
    if workOnCopy==True:
        pdb.gimp_display_new(outImage)

    return(outDrawable)

# This is the plugin registration function
register(
    "filling",    
    "to fill with random colours a single colour area",   
    "This script examines randomly areas with a chosen colour and fill them with random colours.",
    "Diego", 
    "Diego Nassetti ", 
    "2015/2019",
    "filling", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_COLOR, "colore", "Colour to check and fill", (255, 255, 255, 255)),
      (PF_COLOR, "colorr", "Colour to replace residual areas", (127, 127, 127, 255)),
      (PF_OPTION, "search", "Searching method", 1, ("random","sequential")),
      (PF_OPTION, "colours_source", "Colors replacing method", 0, ("random colours",
                                                                   "random colours from chosen palette",
                                                                   "random colours from chosen gradient",
                                                                   "random grays",
                                                                   "random patterns",
                                                                   )),
      (PF_PALETTE, "palette", "Palette name", "Pastels"),
      (PF_GRADIENT, "gradient", "Gradient name", "Tropical Colors"),
      (PF_STRING, "filter", "Patterns names filter"+"\n (make your choice!)", ""),
      (PF_TOGGLE, "usecopy", "Work on a copy", True)
    ], 
    [
      (PF_DRAWABLE, "drawable", "Output drawable", None),
    ],
    filling,
    menu="<Image>/Diego/Render",
    )

main()
